function mpcprops
%MPCPROPS Detailed help on available MPC object properties.
%
%   MPCPROPS gives details on the generic properties of MPC controllers.
%
%   See also MPC, SET, GET.

%	 Author: A. Bemporad
%	 Copyright 1986-2003 The MathWorks, Inc. 
%   $Revision: 1.1.8.3 $  $Date: 2003/12/04 01:32:31 $

DisplayStr = {...
        ' Help on properties of MPC objects.'...
        ' '...
        ' For a prediction model Plant with Ny outputs, Nu inputs, the MPC properties'...
        ' include:'...
        ' '...
        ' Model        - Structure of prediction and estimation models.'...
        '                Model.Plant:       Plant model (LTI object)'...
        '                          Default value: none, must be supplied'...
        '                Model.Disturbance: Model describing color of the input'...
        '                                   disturbances (LTI object).'...
        '                          Default value: integrators'...
        '                Model.Noise:       Model describing color of the plant output'...
        '                                   measurement noise (LTI object)'...
        '                          Default value: static gain (white noise)'...
        '                Model.Nominal:     Structure containing state, input, and output'...
        '                                   values where Model.Plant is linearized.'...
        '                        .X:  State of Model.Plant at operating point'...
        '                        .U:  Input of Model.Plant at operating point'...
        '                        .Y:  Output of Model.Plant at operating point'...
        '                       .DX:  State derivative/update at operating point'...
        '                          Default value: zeros'...
        ' '...
        '    Input signal types are defined in Model.Plant.InputGroup:'...
        '       ManipulatedVariables (or MV or Manipulated or Input):  indices of manipulated variables'...
        '               UnmeasuredDisturbances (or UD or Unmeasured):  indices of unmeasured disturbances'...
        '                   MeasuredDisturbances (or MD or Measured):  indices of measured disturbances'...
        '    Default: unspecified signals are defaulted to ''Manipulated'...   
        ' '...
        '    Output signal types are defined in Model.Plant.OutputGroup:'...
        '                        MeasuredOutputs (or MO or Measured):  indices of measured outputs'...
        '                      UnmeasuredOutputs (or UO or Measured):  indices of unmeasured outputs'...
        '    Default: unspecified signals are defaulted to ''Measured'...   
        ' '...
        ' Ts           - Sampling time of the MPC controller (in seconds).'...
        '    Default: Model.Plant.Ts if this is discrete-time, otherwise must be supplied.'...
        ' '...
        ' PredictionHorizon    - Prediction Horizon (scalar)'...
        '    Default: 10 + max delay in Model.Plant'...
        ' '...
        ' ControlHorizon       - Control Horizon (scalar or vector of blocking moves)'...
        '    Default: 2'...
        ' '...
        ' Weights      - Structure defining MPC weights in the performance index with fields:'...
        '       .ManipulatedVariables (or MV or Manipulated or Input):'...  
        '           Nu x (min 1, max p) matrix of weights on manipulated variables  (default: zeros(Nu,1))'...
        '       .ManipulatedVariablesRate (or MVRate or ManipulatedRate or InputRate):'...
        '           Nu x (min 1, max p) matrix of weights on rates of manipulated varsiables'...
        '           The small quantity 10*sqrt(eps) is automatically added to each weight to ensure'...
        '           numerical stability                                             (default: 0.1*ones(Nu,1))'... 
        '       .OutputVariables (or OV or Output):'...
        '           Ny x (min 1, max p) matrix of weights on controlled outputs     (default: ones(Ny,1))'... 
        '       .ECR: (soft constraints) equal concern relaxation. This is the weight'...
        '           on the scalar slack variable used for constraint softening      (default: 1e5*max(Weights))'... 
        ' '...        
        ' ManipulatedVariables (or MV or Manipulated or Input) - array of structures with fields:'...
        '           .Min(i): 1 to P dimensional vector of lower bounds on MV #i (default: -Inf)'...
        '           .Max(i): 1 to P dimensional vector of upper bounds on MV #i (default: Inf)'...
        '        .MinECR(i): 1 to P dimensional vector describing the equal concern for the relaxation of'...
        '                    lower constraints on MV #i (default: 0)'...
        '        .MaxECR(i): 1 to P dimensional vector describing the equal concern for the relaxation of'...
        '                    upper constraints on MV #i (default: 0)'...
        '        .Target(i): Input variable target value (default: 0)'...
        '       .RateMin(i): 1 to P dimensional vector of lower bounds on rate of MV #i (default: -Inf, -10)'...
        '       .RateMax(i): 1 to P dimensional vector of upper bounds on rate of MV #i (default: Inf)'...
        '    .RateMinECR(i): 1 to P dimensional vector describing the equal concern for the relaxation of'...
        '                    lower constraints on rate of MV #i (default: 0)'...
        '    .RateMaxECR(i): 1 to P dimensional vector describing the equal concern for the relaxation of'...
        '                    upper constraints on rate of MV #i (default: 0)'...
        '          .Name(i): name of MV #i (default: from Model.Plant.InputName'...  
        '         .Units(i): string specifying the measurement units for manipulated variable #i'...  
        ' '...
        '                   (i=1,...,number of manipulated inputs)' ...
        ' '...
        ' OutputVariables (or OV or Controlled or Output) - array of structures with fields:'...
        '           .Min(i): 1 to P dimensional vector of lower bounds on OV #i (default: -Inf)'...
        '           .Max(i): 1 to P dimensional vector of upper bounds on OV #i (default: Inf)'...
        '        .MinECR(i): 1 to P dimensional vector describing the equal concern for the relaxation of'...
        '                    lower constraints on OV #i (default: 0)'...
        '        .MaxECR(i): 1 to P dimensional vector describing the equal concern for the relaxation of'...
        '                    upper constraints on OV #i (default: 0)'...
        '    .Integrator(i): Magnitude of integrated white noise on output channel #i (0=no integrator)'...
        '                    (default: 0 if unmeasured output, 1 if measured output). '...  
        '                    By default, output integrators are added according to '...
        '                    the following rule: (1) outputs are ordered by decreasing output weight'...
        '                    In case of equal output weight, the order within the output vector is followed),'...
        '                    (2) by following such order, an output integrator is added per measured outputs,'...
        '                    unless (a) there is a violation of observability; (b) The corresponding value in'...
        '                    Integrator is zero; (c) the corresponding weight is zero.'...
        '                    Adding integrators on unmeasured output channel is discouraged'...
        '                    See also GETOUTDIST and SETOUTDIST.'...
        '          .Name(i): name of MV #i (default: from Model.Plant.InputName'...  
        '         .Units(i): string specifying the measurement units for manipulated variable #i'...  
        ' '...
        '                   (i=1,...,number of controlled outputs)' ...
        ' '...
        ' DisturbanceVariables (or DV or Disturbance) - array of structures with fields:'...
        '          .Name(i): name of DV #i (default: from Model.Plant.InputName'...  
        '         .Units(i): string specifying the measurement units for disturbance #i'...  
        ' '...
        '                   (i=1,..., total number of measured and unmeasured disturbances)' ...
        ' '...
        ' Optimizer    - QP optimizer parameter structure with fields:'...
        '        .MaxIter:   max number of iterations (default: 200)'...
        '          .Trace:   information verbosity (default: ''off'')'...
        '         .Solver:   QP solver (default: ''ActiveSet'')'...
        '   .MinOutputECR:   min positive value of output MinECR and MaxECR (default: 1e-10)'...
        ' '...
        ' History      - Creation time and related info.'...
        '  '...
        ' Notes        - User''s notes.'...
        '                Content:  Any string or cell array of strings.'...
        '                Example:  ''MPC for constrolling airflow and pressure'''...
        '  '...
        ' Userdata     - Additional information or data.'...
        '                Content:  Any MATLAB data type.'...
        ' '...
        ' Use SET and GET to set and access property values, as in'...
        '    set(mpcobj,''Ts'',''0.1'')   or  mpcobj.Ts=0.1'...
        '    get(mpcobj,''Model'')      or  mpcobj.Model'...
        ' '...
        ' Note: If any finite constraints are specified, then all lower bounds on MV rates should be supplied,' ...  
        '       otherwise the default value -10 is assumed for all unspecified lower bounds on MV rates.' ...
        ' '...
        ' See also MPC.'...
    };


disp(char(DisplayStr))