function [sigma,omega]=svdfrsp(vmat)
%SVDFRSP SVD at each frequency of the frequency response generated by MOD2FRSP
%	[sigma,omega]=svdfrsp(vmat)
%       Given a varying matrix vmat, svdfrsp calculates the singular
%	values of each submatrix in vmat. The output sigma is a matrix
%	of singular values, whose ith row contains the singular values
%	of the ith submatrix in order of decreasing magnitude. The output
%	omega is a column vector containing the independent variable
%	values in vmat.
%
% See also MOD2FRSP.

%       Copyright 1994-2003 The MathWorks, Inc.
%       $Revision: 1.1.6.2 $


if (nargin ~= 1) | (nargout ~= 2)
    disp('Usage : [sigma,omega]=svdfrsp(vmat)')
    return
end
[a b]=size(vmat);
if vmat(a,b) ~= inf
    disp('Error : input needs to be a varying matrix')
    return
end

N=vmat(a,b-1);
omega=vmat(1:N,b);
m=((a-1)/N);
for i=1:N
    sigma(i,:)=(svd(vmat(((i-1)*m+1):i*m,1:(b-1))))';
end
